@@ -142,7 +142,6 @@ struct _GeglTile
 {
  /* GObject          parent_instance;*/
   gint             ref_count;
-
   guchar          *data;        /* actual pixel data for tile, a linear buffer*/
   gint             size;        /* The size of the linear buffer */
 
@@ -181,6 +180,9 @@ struct _GeglTile
                                  * are in the cache.
                                  */
 
+  gint             clones;       // data storage for n_clones
+  gint             cached_clones;//
+
   guint64          damage;
 
   /* called when the tile is about to be destroyed */
