@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås
  */
@@ -22,31 +22,11 @@
 #include <string.h>
 
 #include "gegl.h"
-#include "gegl-types-internal.h"
 #include "gegl-operation-composer.h"
-#include "gegl-utils.h"
-#include "graph/gegl-node.h"
-#include "graph/gegl-connection.h"
-#include "graph/gegl-pad.h"
-#include "buffer/gegl-region.h"
-#include "buffer/gegl-buffer.h"
+#include "gegl-operation-context.h"
+#include "gegl-config.h"
+#include <glib/gi18n-lib.h>
 
-enum
-{
-  PROP_0,
-  PROP_OUTPUT,
-  PROP_INPUT,
-  PROP_AUX
-};
-
-static void     get_property (GObject             *gobject,
-                              guint                prop_id,
-                              GValue              *value,
-                              GParamSpec          *pspec);
-static void     set_property (GObject             *gobject,
-                              guint                prop_id,
-                              const GValue        *value,
-                              GParamSpec          *pspec);
 static gboolean gegl_operation_composer_process (GeglOperation       *operation,
                               GeglOperationContext     *context,
                               const gchar         *output_prop,
@@ -69,41 +49,14 @@ G_DEFINE_TYPE (GeglOperationComposer, gegl_operation_composer,
 static void
 gegl_operation_composer_class_init (GeglOperationComposerClass * klass)
 {
-  GObjectClass       *object_class    = G_OBJECT_CLASS (klass);
   GeglOperationClass *operation_class = GEGL_OPERATION_CLASS (klass);
 
-  object_class->set_property = set_property;
-  object_class->get_property = get_property;
-
   operation_class->process = gegl_operation_composer_process;
+  operation_class->threaded = TRUE;
   operation_class->attach = attach;
   operation_class->detect = detect;
   operation_class->get_bounding_box = get_bounding_box;
   operation_class->get_required_for_output = get_required_for_output;
-
-  g_object_class_install_property (object_class, PROP_OUTPUT,
-                                   g_param_spec_object ("output",
-                                                        "Output",
-                                                        "Output pad for generated image buffer.",
-                                                        GEGL_TYPE_BUFFER,
-                                                        G_PARAM_READABLE |
-                                                        GEGL_PARAM_PAD_OUTPUT));
-
-  g_object_class_install_property (object_class, PROP_INPUT,
-                                   g_param_spec_object ("input",
-                                                        "Input",
-                                                        "Input pad, for image buffer input.",
-                                                        GEGL_TYPE_BUFFER,
-                                                        G_PARAM_READWRITE |
-                                                        GEGL_PARAM_PAD_INPUT));
-
-  g_object_class_install_property (object_class, PROP_AUX,
-                                   g_param_spec_object ("aux",
-                                                        "Input",
-                                                        "Auxiliary image buffer input pad.",
-                                                        GEGL_TYPE_BUFFER,
-                                                        G_PARAM_READWRITE |
-                                                        GEGL_PARAM_PAD_INPUT));
 }
 
 static void
@@ -114,34 +67,74 @@ gegl_operation_composer_init (GeglOperationComposer *self)
 static void
 attach (GeglOperation *self)
 {
-  GeglOperation *operation    = GEGL_OPERATION (self);
-  GObjectClass  *object_class = G_OBJECT_GET_CLASS (self);
+  GeglOperation *operation = GEGL_OPERATION (self);
+  GeglOperationComposerClass *klass = GEGL_OPERATION_COMPOSER_GET_CLASS (operation);
+  GParamSpec    *pspec;
+
+  pspec = g_param_spec_object ("output",
+                               "Output",
+                               "Output pad for generated image buffer.",
+                               GEGL_TYPE_BUFFER,
+                               G_PARAM_READABLE |
+                               GEGL_PARAM_PAD_OUTPUT);
+  gegl_operation_create_pad (operation, pspec);
+  g_param_spec_sink (pspec);
+
+  pspec = g_param_spec_object ("input",
+                               "Input",
+                               "Input pad, for image buffer input.",
+                               GEGL_TYPE_BUFFER,
+                               G_PARAM_READWRITE |
+                               GEGL_PARAM_PAD_INPUT);
+  gegl_operation_create_pad (operation, pspec);
+  g_param_spec_sink (pspec);
+
+  pspec = g_param_spec_object ("aux",
+      klass->aux_label?klass->aux_label:"Aux",
+      klass->aux_description?klass->aux_description:_("Auxiliary image buffer input pad."),
+      GEGL_TYPE_BUFFER,
+      G_PARAM_READWRITE |
+      GEGL_PARAM_PAD_INPUT);
+  gegl_operation_create_pad (operation, pspec);
+  g_param_spec_sink (pspec);
 
-  gegl_operation_create_pad (operation,
-                             g_object_class_find_property (object_class,
-                                                           "output"));
-  gegl_operation_create_pad (operation,
-                             g_object_class_find_property (object_class,
-                                                           "input"));
-  gegl_operation_create_pad (operation,
-                             g_object_class_find_property (object_class,
-                                                           "aux"));
 }
 
-static void
-get_property (GObject    *object,
-              guint       prop_id,
-              GValue     *value,
-              GParamSpec *pspec)
+typedef struct ThreadData
 {
-}
+  GeglOperationComposerClass *klass;
+  GeglOperation              *operation;
+  GeglOperationContext       *context;
+  GeglBuffer                 *input;
+  GeglBuffer                 *aux;
+  GeglBuffer                 *output;
+  const GeglRectangle        *roi;
+  gint                        level;
+  gboolean                    success;
+} ThreadData;
 
 static void
-set_property (GObject      *object,
-              guint         prop_id,
-              const GValue *value,
-              GParamSpec   *pspec)
+thread_process (const GeglRectangle *area,
+                ThreadData          *data)
 {
+  GeglBuffer *input;
+
+  if (gegl_rectangle_equal (area, data->roi))
+    {
+      input = g_object_ref (data->input);
+    }
+  else
+    {
+      input = gegl_operation_context_dup_input_maybe_copy (data->context,
+                                                           "input", area);
+    }
+
+  if (!data->klass->process (data->operation,
+                             input, data->aux, data->output,
+                             area, data->level))
+    data->success = FALSE;
+
+  g_object_unref (input);
 }
 
 static gboolean
@@ -163,9 +156,12 @@ gegl_operation_composer_process (GeglOperation        *operation,
       return FALSE;
     }
 
-  input = gegl_operation_context_get_source (context, "input");
-  aux   = gegl_operation_context_get_source (context, "aux");
-  output = gegl_operation_context_get_target (context, "output");
+  input = (GeglBuffer*)gegl_operation_context_dup_object (context, "input");
+  aux   = (GeglBuffer*)gegl_operation_context_dup_object (context, "aux");
+  output = gegl_operation_context_get_output_maybe_in_place (operation,
+                                                             context,
+                                                             input,
+                                                             result);
 
   /* A composer with a NULL aux, can still be valid, the
    * subclass has to handle it.
@@ -173,20 +169,41 @@ gegl_operation_composer_process (GeglOperation        *operation,
   if (input != NULL ||
       aux != NULL)
     {
-      success = klass->process (operation, input, aux, output, result, level);
+      if (gegl_operation_use_threading (operation, result))
+      {
+        ThreadData data;
 
-      if (output == GEGL_BUFFER (operation->node->cache))
-        gegl_cache_computed (operation->node->cache, result);
+        data.klass = klass;
+        data.operation = operation;
+        data.context = context;
+        data.input = input;
+        data.aux = aux;
+        data.output = output;
+        data.roi = result;
+        data.level = level;
+        data.success = TRUE;
 
-      if (input)
-        g_object_unref (input);
-      if (aux)
-        g_object_unref (aux);
+        gegl_parallel_distribute_area (
+          result,
+          gegl_operation_get_pixels_per_thread (operation),
+          GEGL_SPLIT_STRATEGY_AUTO,
+          (GeglParallelDistributeAreaFunc) thread_process,
+          &data);
+
+        success = data.success;
+      }
+      else
+      {
+        success = klass->process (operation, input, aux, output, result, level);
+      }
+
+      g_clear_object (&input);
+      g_clear_object (&aux);
     }
   else
     {
       g_warning ("%s received NULL input and aux",
-                 gegl_node_get_debug_name (operation->node));
+                 gegl_node_get_operation (operation->node));
     }
 
   return success;
@@ -199,20 +216,12 @@ get_bounding_box (GeglOperation *self)
   GeglRectangle *in_rect  = gegl_operation_source_get_bounding_box (self, "input");
   GeglRectangle *aux_rect = gegl_operation_source_get_bounding_box (self, "aux");
 
-  if (!in_rect)
-    {
-      if (aux_rect)
-        return *aux_rect;
-      return result;
-    }
+  if (in_rect)
+    result = *in_rect;
+
   if (aux_rect)
-    {
-      gegl_rectangle_bounding_box (&result, in_rect, aux_rect);
-    }
-  else
-    {
-      return *in_rect;
-    }
+    gegl_rectangle_bounding_box (&result, &result, aux_rect);
+
   return result;
 }
 
