@@ -1,5 +1,5 @@
 /* This file is an image processing operation for GEGL
-* 
+*
 * GEGL is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
@@ -28,8 +28,8 @@ property_enum (curve, _("Characteristic curve"),
 
 property_double (exposure, _("Exposure"), 0.0)
 	description(_("Base enlargement exposure"))
-	value_range (-10, 10)
-	ui_range (-5, 5)
+	value_range (-20, 10)
+	ui_range (-15, 5)
 
 property_double (expC, _("Cyan filter"), 60)
 	description(_("Cyan filter compensation for the negative image"))
@@ -48,17 +48,42 @@ property_double (expY, _("Yellow filter"), 60)
 
 property_boolean (clip, _("Clip base + fog"), TRUE)
 	description (_("Clip base + fog to have a pure white output value"))
+property_double (add_fog, _("Add base and fog"), 0.0)
+	description (_("Artificially reintroduce base and fog."))
+	ui_meta("visible", "clip")
+	value_range (-2, 4)
+	ui_range (0, 2)
 
+property_double (boost_c, _("Cyan density boost"), 1.0)
+	description(_("Boost paper density to take advantage of increased dynamic range of a monitor compared to a photographic paper"))
+	value_range (0.25, 10)
+	ui_range (1, 4)
+	ui_gamma (2)
 property_double (boost, _("Density boost"), 1.0)
 	description(_("Boost paper density to take advantage of increased dynamic range of a monitor compared to a photographic paper"))
+	value_range (0.25, 10)
+	ui_range (1, 4)
+	ui_gamma (2)
+property_double (boost_y, _("Yellow density boost"), 1.0)
+	description(_("Boost paper density to take advantage of increased dynamic range of a monitor compared to a photographic paper"))
+	value_range (0.25, 10)
+	ui_range (1, 4)
+	ui_gamma (2)
+property_double (contrast_r, _("Contrast boost R"), 1.0)
+	description(_("Increase red contrast for papers with fixed contrast (usually color papers)"))
 	value_range (0.25, 4)
-	ui_range (1, 2)
+	ui_range (0.75, 1.5)
 	ui_gamma (2)
 property_double (contrast, _("Contrast boost"), 1.0)
 	description(_("Increase contrast for papers with fixed contrast (usually color papers)"))
 	value_range (0.25, 4)
 	ui_range (0.75, 1.5)
 	ui_gamma (2)
+property_double (contrast_b, _("Contrast boost B"), 1.0)
+	description(_("Increase blue contrast for papers with fixed contrast (usually color papers)"))
+	value_range (0.25, 4)
+	ui_range (0.75, 1.5)
+	ui_gamma (2)
 
 property_double (dodge, _("Dodge/burn multiplier"), 1.0)
 	description(_("The f-stop of dodge/burn for pure white/black auxiliary input"))
@@ -99,7 +124,7 @@ property_double (illumZ, _("Z multiplier"), 0.829)
 #define GEGL_OP_NAME     negative_darkroom
 #define GEGL_OP_C_SOURCE negative-darkroom.c
 
-#define EPSILON 0.00001
+#define EPSILON 0.00001f
 
 #include "gegl-op.h"
 
@@ -152,15 +177,28 @@ curve_lerp (gfloat * xs, gfloat * ys, guint n, gfloat in)
 {
 	if (in <= xs[0])
 		return(ys[0]);
-	for (guint i = 1; i <= n; i++)
+
+	guint min = 0;
+	guint max = n - 1;
+
+	if (in >= xs[max])
+		return(ys[max]);
+
+	// Binary tree search
+	guint split;
+	while (max - min > 1)
 	{
-		if (in <= xs[i])
+		split = min + ((max - min) / 2);
+		if (in < xs[split])
 		{
-			return(ys[i-1] + (in - xs[i-1]) * \
-			       ((ys[i] - ys[i-1]) / (xs[i] - xs[i-1])));
+			max = split;
 		}
-	}	
-	return(ys[n-1]);
+		else
+		{
+			min = split;
+		}
+	}
+	return(ys[min] + (in - xs[min]) * ((ys[max] - ys[min]) / (xs[max] - xs[min])));
 }
 
 static gfloat
@@ -208,33 +246,34 @@ process (GeglOperation       *operation,
 	gfloat *aux  = aux_buf;
 	gfloat *out  = out_buf;
 
-	gfloat Dfogc = 0;
-	gfloat Dfogm = 0;
-	gfloat Dfogy = 0;
+	gfloat Dfogc = 0.f;
+	gfloat Dfogm = 0.f;
+	gfloat Dfogy = 0.f;
 
-	gfloat rcomp = 0;
-	gfloat gcomp = 0;
-	gfloat bcomp = 0;
+	gfloat rcomp = 0.f;
+	gfloat gcomp = 0.f;
+	gfloat bcomp = 0.f;
 
-	gfloat r = 0;
-	gfloat g = 0;
-	gfloat b = 0;
+	gfloat r = 0.f;
+	gfloat g = 0.f;
+	gfloat b = 0.f;
 
-	gfloat x = 0;
-	gfloat y = 0;
-	gfloat z = 0;
+	gfloat x = 0.f;
+	gfloat y = 0.f;
+	gfloat z = 0.f;
+
+	gfloat exp = powf(2.f, o->exposure);
 
-	gfloat exp = pow(2, o->exposure);
 
 	// Calculate base+fog
 	if (o->clip)
 	{
 		Dfogc = array_min(curves[o->curve].ry,
-				  curves[o->curve].rn) * o->boost;
+				  curves[o->curve].rn) * o->boost * o->boost_c;
 		Dfogm = array_min(curves[o->curve].gy,
 				  curves[o->curve].gn) * o->boost;
 		Dfogy = array_min(curves[o->curve].by,
-				  curves[o->curve].bn) * o->boost;
+				  curves[o->curve].bn) * o->boost * o->boost_y;
 	}
 
 	// Calculate exposure for mid density
@@ -244,53 +283,56 @@ process (GeglOperation       *operation,
 	gfloat rMid = curve_lerp(curves[o->curve].ry,
 				 curves[o->curve].rx,
 				 curves[o->curve].rn,
-				 Dmaxc / 2);
+				 Dmaxc / 2.f);
 	gfloat gMid = curve_lerp(curves[o->curve].gy,
 				 curves[o->curve].gx,
 				 curves[o->curve].gn,
-				 Dmaxm / 2);
+				 Dmaxm / 2.f);
 	gfloat bMid = curve_lerp(curves[o->curve].by,
 				 curves[o->curve].bx,
 				 curves[o->curve].bn,
-				 Dmaxy / 2);
+				 Dmaxy / 2.f);
 
 	if (!aux)
 	{
-		rcomp = pow(2, (-o->expC) / 30);
-		gcomp = pow(2, (-o->expM) / 30);
-		bcomp = pow(2, (-o->expY) / 30);
+		rcomp = powf(2.f, (-o->expC) / 30.f);
+		gcomp = powf(2.f, (-o->expM) / 30.f);
+		bcomp = powf(2.f, (-o->expY) / 30.f);
 	}
 
 	for (glong i = 0; i < n_pixels; i++)
 	{
-		/*printf("Input XYZ intensity %f %f %f\n", in[0], in[1], in[2]);*/
+		/*printf("==\nInput XYZ intensity %f %f %f\n", in[0], in[1], in[2]);*/
 
 		// Calculate exposure compensation from global+filter+dodge
 		if (aux)
 		{
-			rcomp = pow(2, ((-o->expC) / 30) -
-					(2 * o->dodge * (aux[0] - 0.5)));
-			gcomp = pow(2, ((-o->expM) / 30) -
-					(2 * o->dodge * (aux[1] - 0.5)));
-			bcomp = pow(2, ((-o->expY) / 30) -
-					(2 * o->dodge * (aux[2] - 0.5)));
+			rcomp = powf(2.f, ((-o->expC) / 30.f) -
+					(2.f * o->dodge * (aux[0] - 0.5f)));
+			gcomp = powf(2.f, ((-o->expM) / 30.f) -
+					(2.f * o->dodge * (aux[1] - 0.5f)));
+			bcomp = pow(2.f, ((-o->expY) / 30.f) -
+					(2.f * o->dodge * (aux[2] - 0.5f)));
 			aux  += 3;
 		}
 
 		// Convert to CIERGB primaries for color filter balance
-		x =    0.41847*in[0] -   0.15866*in[1] - 0.082835*in[2];
-		y =  -0.091169*in[0] +   0.25243*in[1] + 0.015708*in[2];
-		z = 0.00092090*in[0] - 0.0025498*in[1] +  0.17860*in[2];
+		x =    0.41847f*in[0] -   0.15866f*in[1] - 0.082835f*in[2];
+		y =  -0.091169f*in[0] +   0.25243f*in[1] + 0.015708f*in[2];
+		z = 0.00092090f*in[0] - 0.0025498f*in[1] +  0.17860f*in[2];
+		/*printf("Input CIERGB intensity %f %f %f\n", x, y, z);*/
 
 		// Apply preflash
-		x += o->flashC / 100;
-		y += o->flashM / 100;
-		z += o->flashY / 100;
+		x += o->flashC / 100.f;
+		y += o->flashM / 100.f;
+		z += o->flashY / 100.f;
+		/*printf("Preflashed intensity %f %f %f\n", x, y, z);*/
 
 		// Apply color filters and exposure
 		x *= rcomp * exp;
 		y *= gcomp * exp;
 		z *= bcomp * exp;
+		/*printf("After color balance and exposure %f %f %f\n", x, y, z);*/
 
 		// Simulate emulsion spectral sensitivity with
 		// sensitivity matrix
@@ -305,20 +347,22 @@ process (GeglOperation       *operation,
 			   z * curves[o->curve].bsens.Z);
 
 		// Scale the emulsion response
-		r *= 5000;
-		g *= 5000;
-		b *= 5000;
+		r *= 5000.f;
+		g *= 5000.f;
+		b *= 5000.f;
+		/*printf("Linear RGB intensity %f %f %f\n", r, g, b);*/
 
 		// Logarithmize the input
-		r = log10(r);
-		g = log10(g);
-		b = log10(b);
+		r = log10f(r);
+		g = log10f(g);
+		b = log10f(b);
 		/*printf("Logarithmic RGB intensity %f %f %f\n", r, g, b);*/
 
 		// Adjust contrast
-		r = (r - rMid) * o->contrast + rMid;
+		r = (r - rMid) * o->contrast * o->contrast_r + rMid;
 		g = (g - gMid) * o->contrast + gMid;
-		b = (b - bMid) * o->contrast + bMid;
+		b = (b - bMid) * o->contrast * o->contrast_b + bMid;
+		/*printf("Logarithmic RGB intensity after contrast %f %f %f\n", r, g, b);*/
 
 		// Apply the DH curve
 		r = curve_lerp(curves[o->curve].rx,
@@ -336,9 +380,16 @@ process (GeglOperation       *operation,
 		/*printf("Raw RGB density %f %f %f\n", r, g, b);*/
 
 		// Apply density boost
-		r *= o->boost;
+		r *= o->boost * o->boost_c;
 		g *= o->boost;
-		b *= o->boost;
+		b *= o->boost * o->boost_y;
+		/*printf("Raw RGB density after density boost %f %f %f\n", r, g, b);*/
+
+		// Apply artificial fog
+		r += o->add_fog;
+		g += o->add_fog;
+		b += o->add_fog;
+		/*printf("Raw RGB density after fogging %f %f %f\n", r, g, b);*/
 
 		// Compensate for fog
 		r -= Dfogc;
@@ -347,16 +398,16 @@ process (GeglOperation       *operation,
 		/*printf("Adjusted RGB density %f %f %f\n", r, g, b);*/
 
 		// Simulate dye density with exponentiation to get
-		// the CIEXYZ transmittance back
-		out[0] = pow(10, -(r * curves[o->curve].cdens.X +
-				   g * curves[o->curve].mdens.X +
-				   b * curves[o->curve].ydens.X)) * o->illumX;
-		out[1] = pow(10, -(r * curves[o->curve].cdens.Y +
-				   g * curves[o->curve].mdens.Y +
-				   b * curves[o->curve].ydens.Y));
-		out[2] = pow(10, -(r * curves[o->curve].cdens.Z +
-				   g * curves[o->curve].mdens.Z +
-				   b * curves[o->curve].ydens.Z)) * o->illumZ;
+		// the CIEXYZ tramsmittance back
+		out[0] = (1 / powf(10.f, r * curves[o->curve].cdens.X)) *
+			 (1 / powf(10.f, g * curves[o->curve].mdens.X)) *
+			 (1 / powf(10.f, b * curves[o->curve].ydens.X)) * o->illumX;
+		out[1] = (1 / powf(10.f, r * curves[o->curve].cdens.Y)) *
+			 (1 / powf(10.f, g * curves[o->curve].mdens.Y)) *
+			 (1 / powf(10.f, b * curves[o->curve].ydens.Y));
+		out[2] = (1 / powf(10.f, r * curves[o->curve].cdens.Z)) *
+			 (1 / powf(10.f, g * curves[o->curve].mdens.Z)) *
+			 (1 / powf(10.f, b * curves[o->curve].ydens.Z)) * o->illumZ;
 		/*printf("XYZ output %f %f %f\n", out[0], out[1], out[2]);*/
 
 		in   += 3;
@@ -385,8 +436,8 @@ gegl_op_class_init (GeglOpClass *klass)
 		"name",           "gegl:negative-darkroom",
 		"title",          _("Negative Darkroom"),
 		"categories",     "color",
-		"reference-hash", "81752f5612f3f639f68522fae6c506a3",
-		"description",    _("Simulate a negative film enlargement in "
+		"reference-hash", "unstable",
+		"description",    _("Simulate a film enlargement in "
 				  "an analog darkroom."),
 		NULL);
 }
