@@ -33,8 +33,8 @@ TEST ()
 
   fill (subsub, 1.0);
   print_buffer (buffer);
-  gegl_buffer_destroy (sub);
-  gegl_buffer_destroy (subsub);
-  gegl_buffer_destroy (buffer);
+  g_object_unref (sub);
+  g_object_unref (subsub);
+  g_object_unref (buffer);
   test_end ();
 }
